/* PhyloPattern
 * Copyright (C) 2005-2009 Philippe Gouret
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public
 * License along with this library; if not, write to the
 * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * Boston, MA  02111-1307, USA. The text ol license can be also found 
 * at http://www.gnu.org/copyleft/gpl.html
 */

//------------------------------------------------------------------------------
// Stream transfer Implementation
// Philippe Gouret
//------------------------------------------------------------------------------

/**
 *  Signal
 *  @since JDK1.2
 */
package gnu.prolog.vm.buildins.PhyloPattern;

//imported classes
import java.io.*;

//project classes

/**
 *  Stream transfer (read from one write to other) Implementation class
 *  @author Philippe Gouret
 */
public class StreamTransfer implements Runnable
{
	//----------------------------------------------------------------------------
	// Constants
	//----------------------------------------------------------------------------

	// buffer size
	static private final int READBUFFER_SIZE = 1024;

	//----------------------------------------------------------------------------
	// Public members
	//----------------------------------------------------------------------------

	//----------------------------------------------------------------------------
	// Private members
	//----------------------------------------------------------------------------

	// buffer used for read
	private char[] buffer = new char[READBUFFER_SIZE];

	// label
	private String label;

	// Reader
	private Reader in;

	// Writer
	private Writer out;

	// stream transfered
	private boolean transfered = false;

	// close input after transfer
	private boolean closeInput = false;

	//----------------------------------------------------------------------------
	// Private methods
	//----------------------------------------------------------------------------

	//----------------------------------------------------------------------------
	// Public methods
	//----------------------------------------------------------------------------

	/**
	 *  run
	 */
	public void run()
	{
		try
		{
			// buffered
			BufferedReader inBuffered = new BufferedReader(in);
			BufferedWriter outBuffered = new BufferedWriter(out);

			// transfer
			int n;
			while ((n = inBuffered.read(buffer, 0, READBUFFER_SIZE)) != -1)
			{
				Thread.yield();
				outBuffered.write(buffer, 0, n);
			}
			outBuffered.flush();
			if (closeInput) in.close();
		}
		catch(IOException e)
		{
			// not transfered
			transfered = false;
		}

		// transfered
		transfered = true;
	}

	/**
	 *  is transfered
	 *  @return fact
	 */
	public boolean isTransfered()
	{
		return transfered;
	}

	/**
	 *  constructor
	 *  @param text used for trace
	 *  @param input stream
	 *  @param output stream
	 */
	public StreamTransfer(String label, Reader in, Writer out)
	{
		// keep info
		this.label = label;
		this.in = in;
		this.out = out;
	}

	/**
	 *  constructor
	 *  @param text used for trace
	 *  @param input stream
	 *  @param output stream
	 *  @param close input
	 */
	public StreamTransfer(String label, Reader in, Writer out, boolean closeInput)
	{
		// keep info
		this.label = label;
		this.in = in;
		this.out = out;
		this.closeInput = closeInput;
	}
}


