%  PhyloPattern
%  Copyright (C) 2005-2009 Philippe Gouret
%  This library is free software; you can redistribute it and/or
%  modify it under the terms of the GNU General Public
%  License as published by the Free Software Foundation; either
%  version 2 of the License, or (at your option) any later version.
%  This library is distributed in the hope that it will be useful,
%  but WITHOUT ANY WARRANTY; without even the implied warranty of
%  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
%  General Public License for more details.
% 
%  You should have received a copy of the GNU General Public
%  License along with this library; if not, write to the
%  Free Software Foundation, Inc., 59 Temple Place - Suite 330,
%  Boston, MA  02111-1307, USA. The text ol license can be also found 
%  at http://www.gnu.org/copyleft/gpl.html
%

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%	Samples
% (NHX files used in tha samples can be visualized by Rio Forester Java GUI)
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%	Tools
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

% take bootstrap values in one label
bootstraps(Label, Bootstraps) :- bootstrap_label(Label, Bootstraps), !.
bootstraps(_, [0]).

% good bootstraps
good_bootstraps(Tags) :- element(name(Label), Tags), bootstraps(Label, Bootstraps), element(Bootstrap, Bootstraps), Bootstrap >= 90, !.
good_bootstraps(Tags) :-
	element(name(Label), Tags), bootstraps(Label, Bootstraps),
	element(Bootstrap1, Bootstraps), without(Bootstrap1, Bootstraps, WBootstraps), element(Bootstrap2, WBootstraps),
	Bootstrap1 >= 75, Bootstrap2 >= 60, !.
good_bootstraps(Tags) :-
	element(name(Label), Tags), bootstraps(Label, Bootstraps),
	element(Bootstrap1, Bootstraps), without(Bootstrap1, Bootstraps, WBootstraps), element(Bootstrap2, WBootstraps),
	without(Bootstrap2, WBootstraps, WWBootstraps), element(Bootstrap3, WWBootstraps),
	Bootstrap1 >= 50, Bootstrap2 >= 50,	Bootstrap3 >= 50, !.

% 
report_matching_trees([], _).
report_matching_trees([Tree|Trees], RefTree) :-
	pattern_matching(RefTree, @(Tree), [], Res),
	nhx(Res, NHX),
	qprintln('-----------------------------------------------------'),
	qprintln(NHX),
	report_matching_trees(Trees, RefTree).
	
further([[], Tags1], [[], Tags2]) :-
	element(dist(Dist1), Tags1),
	element(dist(Dist2), Tags2),
	Dist1 > Dist2.
	
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Tree comparing, we extract the list of first tree's subtrees that matches
% with a subtree of the second tree
% just call from the shell: prompt> run_... test_tree_comparing
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
		
% test tree comparing: load two trees, keep only wired structure for comparing
% break bad nodes to rakes, give matching trees 
test_tree_comparing :-
	load_file_as_atom('trees/vset.nhx', Tree1),
	load_file_as_atom('trees/C2set.nhx', Tree2),
	nhx_as_term(Tree1, Term1),
	nhx_as_term(Tree2, Term2),
	wired_and_broken_tree(Term1, Broken1),
	wired_and_broken_tree(Term2, Broken2),
	matching_trees(Broken1, Broken2, Res, false), !,
	report_matching_trees(Res, Term1).

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Tree annotation, here we set for each node its distance to the node with
% the NHX tag :ID=42, then we select the node at the shortest distance.
% just call from the shell: prompt> run_... test_annotation
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

% test tree annotation by setting a topological distance to a specific node:
% load a tree, annotate with distance, sort to keep the shortest
test_annotation :-
	load_file_as_atom('trees/vset.nhx', Tree),
	nhx_as_term(Tree, Term),
	annotate_tree(Term, topological_distance2(['42']), Result, postorder),
	leaf_list(Result, Leafs),
	quick_sort(Leafs, further, [Nearest|_]),
	qprintln_list(Nearest).
	
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%	Pattern matching for orthologs searching.
%	just call from the shell: prompt> run_... "test_pattern_matching(Ortholog)"
%	type 'y' to get next solution
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

% orthologs searching for an input sequence
% +TreeFile:			a file containing a tree with NHX format (see Rio Forester)
% +SequenceName:		name of sequence which one search orthologs for
% +SequenceSpeciesName:	sequence species name
% -OrthologTags:		result (predicate backtracks and give all solutions)
%
ortholog(TreeFile, SequenceName, SequenceSpeciesName, OrthologTags) :-
	load_file_as_atom(TreeFile, Tree),
	nhx_as_term(Tree, Term),
	pattern_matching(Term, @([[@([[], OrthologTags]), @([[], SequenceTags])], ParentTags]),
		[
			freeze([SequenceTags, element(name(SequenceName), SequenceTags)]),
			freeze([OrthologTags, element(species(OrthologSpeciesName), OrthologTags)]),
			freeze([SequenceTags, diff(OrthologSpeciesName, SequenceSpeciesName)]),
			freeze([ParentTags, element(duplication(false), ParentTags)])
		], _).

% test pattern matching: load a tree, apply an orthology pattern
% -Result:	ortholog in the tree
test_pattern_matching(Result) :-
	ortholog(
			'trees/vset.nhx',
			'lcl|ENSP00000383836|9606|HOMO~SAPIENS|',
			'HOMO~SAPIENS',
			Result).
			
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

